import java.awt.Dimension;
import java.awt.event.KeyEvent;
import java.awt.event.KeyAdapter;
import java.awt.event.WindowEvent;
import java.awt.event.WindowAdapter;
import java.awt.Color;
import java.awt.FileDialog;
import java.awt.Font;

import java.io.OutputStream;
import java.io.PrintWriter;

import java.util.Hashtable;

import javax.swing.JFrame;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.UIManager;
import javax.swing.UnsupportedLookAndFeelException;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;

import javax.imageio.ImageIO;

import java.awt.color.ColorSpace;
import java.awt.color.ICC_ColorSpace;
import java.awt.color.ICC_Profile;

public class TurtleArt {

  static LogoConsole logoconsole;
  static LContext userlogo;

  static ColorSpace cmykcs;
  static boolean cmykenabled = false;
  static String iccname;

  static String[] primclasses =
    {"SystemPrims", "MathPrims", "ControlPrims",
     "DefiningPrims", "WordListPrims", "CCPrims", "FilePrims",
      "SoundPrims", "TurtlePrims"};

  static int dpiscale=2;
  static String[] UIStrings = {"TurtleArt", "Yes", "No", "Cancel", "Save", "Open", "Help", "Alert"};

  public static void main(String[] args){
    int index = args.length;
    boolean showcc = false;
    String project = "";
    if((index>0)&&args[index-1].equals("cc")) {showcc = true; index--;}
    (logoconsole = new LogoConsole("Command Center")).init(showcc);
    if(index>0) project=args[0];
    if((project.length()>0) && project.substring(0,1).equals("~")) decodeArg(project);
    else logoconsole.cc.lc.projectfile=project;
    new Thread(new LogoCommandRunner("load \"sprites startup", logoconsole.cc.lc, true)).start();
    logoconsole.tyo.println("Welcome to Logo!");
    logoconsole.cc.requestFocus();
    userlogo = new LContext();
    userlogo.tyo = logoconsole.tyo;
    Logo.setupPrims(primclasses, userlogo);
    String s = System.getProperty("os.name").toLowerCase();
    if(s.indexOf("mac")==-1) return;
    try {Class.forName("PicoApplication").newInstance();} // so it can compile on windows
    catch (Exception e) {System.out.println(e);}
    }

  static void decodeArg(String s){
    s = s.substring(1);
    int sep = s.indexOf("~");
    if(sep>-1){
      cmykenabled = true;
      String fname = s.substring(0,sep)+".icc";
      try {
        cmykcs = new ICC_ColorSpace(ICC_Profile.getInstance(fname));
        iccname = fname;
      }
      catch (Exception e) {System.out.println(e);}
      s = s.substring(sep+1);
    }
    dpiscale = Integer.parseInt(s);
  }
}

class LogoConsole extends JFrame {

  Listener cc;
  static PrintWriter tyo;

  LogoConsole(String s){
     super(s);
     try {UIManager.setLookAndFeel( UIManager.getSystemLookAndFeelClassName() );}
     catch( Exception ex ) {System.out.println( "Look and Feel: " + ex.getLocalizedMessage() );}
     setDefaultCloseOperation (DO_NOTHING_ON_CLOSE);
    }

  static String[] primclasses =
    {"SystemPrims", "MathPrims", "ControlPrims",
     "DefiningPrims", "WordListPrims", "CCPrims",
      "FilePrims","SpritePrims", "WidgetPrims",
      "SoundPrims", "UserlogoPrims", "TCPPrims"};

  void init(boolean showcc) {
    cc=new Listener(40, 40);

    cc.setFont(new Font("Courier", Font.PLAIN, 12));
    tyo = new PrintWriter(new TEStream(cc), true);
// added because of swing
    JScrollPane scrollPane = new JScrollPane(cc,
              JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
              JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED );
    getContentPane().add("Center", scrollPane);
// end
    pack();
 //
   if(showcc) setVisible(true);
    cc.lc = new LContext();
    addWindowListener(new ToplevelWindowListener(cc.lc));
    cc.addKeyListener(new TEKeyListener() );
    cc.lc.tyo = tyo;
    Logo.setupPrims(primclasses, cc.lc);
  }
}


class ToplevelWindowListener extends WindowAdapter
{
  LContext lc;

  ToplevelWindowListener(LContext lc){
      super();
      this.lc=lc;
      }

  public void windowClosing(WindowEvent e) {
    exitApp();
  }

  void exitApp(){
    Object [] listtorun = new Object [1];
    listtorun[0] = Logo.intern("windowclose", lc);
    (new LogoCommandRunner(listtorun, lc, true)).run();
    if (lc.shouldExit) SwingUtilities.invokeLater(new ExitCommand());
  }
}

class ExitCommand implements Runnable {
  public void run() {System.exit(0);}
}

class TEKeyListener extends KeyAdapter{
  public void keyPressed(KeyEvent e){
    Listener cc = (Listener) e.getComponent();
    char key = e.getKeyChar();
    int code = e.getKeyCode();                          // Patch so Ctrl + C is not interpreted as a return
    if(key== '\n' && code == 10){cc.handlecr(); e.consume(); return;}
    if(key=='\u0001'){cc.selectAll(); e.consume(); return;}
    if (key=='\u0012') {cc.runLine("reload startup"); e.consume(); return;} // ctrl+r
    }
}

class LCKeyListener extends KeyAdapter{

  LContext lc;

    LCKeyListener(LContext lc){
    this.lc = lc;
  }

  public void keyPressed (KeyEvent e){
    lc.canvas.keyPressed (e);
    }

  public void keyReleased (KeyEvent e){
    lc.canvas.keyReleased (e);
     }

}


class Listener extends JTextArea {

  LContext lc;
  String file, dir;
  String prefix="", suffix="";

  Listener(int h, int w){super(h,w);};

  void handlecr(){
    String s=getText();
    int sol=findStartOfLine(s, getCaretPosition());
    int eol=findEndOfLine(s, sol);
    if(eol==s.length()) append("\n");
    setCaretPosition(eol+1);
    if(lc.thread!=null){lc.timeToStop=true; return;}
    if(!prefix.equals("")) runSilent(prefix+s.substring(sol, eol)+suffix);
    else runLine(s.substring(sol, eol));
  }

  int findStartOfLine(String s, int i){
    int val = s.lastIndexOf(10,i-1);
    if (val<0) return 0;
    return val+1;
  }

  int findEndOfLine(String s, int i){
    int val = s.indexOf('\n',i);
    if (val<0) return s.length();
    return val;
  }

  void runLine(String s){
    if (lc.thread==null) new Thread(new LogoCommandRunner(s, lc)).start();
  }

  void runSilent(String s){
    if (lc.thread==null){
      new Thread(new LogoCommandRunner(s, lc, true)).start();
    } else {
      lc.timeToStop = true;
    }
  }

}

class TEStream extends OutputStream {

  JTextArea te;
  String buffer = "";

  public TEStream(JTextArea te){this.te = te;}

  public void write(int n) {
    if (n==10) buffer+='\n';
    else if (n==13) return;
    else buffer+=(char)n;
  }

  public void flush(){
    int pos = te.getCaretPosition();
    te.insert(buffer, pos);
    te.setCaretPosition(pos+buffer.length());
    buffer="";
  }
}

class CCPrims extends Primitives {

  static String[] primlist={
    "print", "1", "projectfile", "0"
};

  public String[] primlist(){return primlist;}

  public Object dispatch(int offset, Object[] args, LContext lc){
    switch(offset){
    case 0: return prim_print(args[0], lc);
    case 1: return lc.projectfile;
 }
    return null;
  }

  Object prim_print(Object arg1, LContext lc){
    lc.tyo.println(Logo.prs(arg1));
    return null;
  }
}

class LContext {
  Hashtable<String,Symbol> oblist = new Hashtable<String,Symbol>();
  Hashtable<Object,Hashtable<Object,Object>> props = new Hashtable<Object,Hashtable<Object,Object>>();
  MapList iline;
  Symbol cfun, ufun;
  boolean mustOutput, timeToStop, shouldExit;
  int priority=0, level=0;
  Object juststop = new Object();
  Object ufunresult;
  Object[] locals;
  PrintWriter tyo;
  Thread thread;
  String filename, errormessage, projectfile="";
  SpriteFrame frame;
  SpriteCanvas canvas;
  SpritePanel panel;
  Sprite turtle;
  boolean shiftdown, controldown, altdown;
  int clickcount;
}

